use std::{error::Error, fs::File, io::Write};
use clap::Parser;
use crate::reskit::{tileset, soundtrack::{formats::dmf::DmfModule, engines::echo::engine::EchoFormat}, utility::print_good};
use super::settings::{Args, Tools, TileOutputFormat, TileOrder};

pub fn run_command() -> Result<(), Box<dyn Error>> {
    let cli = Args::parse();

    match cli.tool {
        Tools::Tileset { input_file, output_file, format, tile_order } =>
            tileset::generate(
                &input_file,
                &output_file,
                match format {
                    TileOutputFormat::Bin => "bin",
                    TileOutputFormat::Inc => "inc"
                },
                match tile_order {
                    TileOrder::Tile => "tile",
                    TileOrder::Sprite => "sprite"
                }
            ),
        Tools::Soundtrack { input_file, output_file, input_format: _, output_format: _, source_file_format: _, artifact_output_directory, soundtrack_label, instrument_list_label } => {
            let result = DmfModule::from_file( &input_file )?;

            for ( filename, data ) in result.get_artifacts( &soundtrack_label, &output_file, &artifact_output_directory, &instrument_list_label )? {
                println!( "Creating artifact file {}{}", artifact_output_directory, filename );
                let mut file = File::create( format!( "{}{}", artifact_output_directory, filename ) )?;
                file.write_all( &data )?
            }

            println!( "Creating ESF output file {}", output_file );
            let mut file = File::create( &output_file )?;
            file.write_all( &result.get_esf()? )?;

            print_good( &format!( "successfully compiled soundtrack for {}", output_file ) );
        }
    };

    Ok( () )
}