use clap::{Parser, Subcommand, ValueEnum};

#[derive(Parser)]
#[command(name = "reskit")]
#[command(version = env!("CARGO_PKG_VERSION"))]
#[command(long_about = "reskit\nThe Retro Entertainment Software Toolkit - Utilities for authoring software for legacy video game systems\n(c) 2023 Ashley N. (ne0ndrag0n)")]
pub struct Args {

    #[command(subcommand)]
    pub tool: Tools

}

#[derive(Clone, ValueEnum)]
pub enum TileOrder {
    Tile,
    Sprite
}

#[derive(Clone, ValueEnum)]
pub enum TileOutputFormat {
    Bin,
    Inc
}

#[derive(Clone, ValueEnum)]
pub enum SequenceFormat {
    Dmf
}

#[derive(Clone, ValueEnum)]
pub enum DriverFormat {
    Esf
}

#[derive(Clone, ValueEnum)]
pub enum ArtifactListFormat {
    Asmx
}

#[derive(Subcommand)]
pub enum Tools {

    #[command(name = "tileset")]
    #[command(about = "Generate a set of tiles from an input image. Supports Sega Mega Drive VDP output format.")]
    Tileset {
        /// Input filename
        #[arg(short, long)]
        input_file: String,

        /// Output filename
        #[arg(short, long)]
        output_file: String,

        /// Output format
        #[arg(short, long, value_enum, default_value_t=TileOutputFormat::Bin)]
        format: TileOutputFormat,

        /// Tile order
        #[arg(short, long, value_enum, default_value_t=TileOrder::Tile)]
        tile_order: TileOrder
    },

    #[command(name = "soundtrack")]
    #[command(about = "Generate a console-compatible soundtrack from a sequence file.")]
    Soundtrack {
        /// Input filename
        #[arg(short, long)]
        input_file: String,

        /// Output filename
        #[arg(short, long)]
        output_file: String,

        /// Input sequence file format (the kind of tracker used to compose the track)
        #[arg(long, value_enum, default_value_t=SequenceFormat::Dmf)]
        input_format: SequenceFormat,

        /// Output console file format (the sound driver your console uses)
        #[arg(long, value_enum, default_value_t=DriverFormat::Esf)]
        output_format: DriverFormat,

        /// Output format for the generated source file linking soundtracks and instruments
        #[arg(long, value_enum, default_value_t=ArtifactListFormat::Asmx)]
        source_file_format: ArtifactListFormat,

        /// Directory to output artifacts (instruments and samples)
        #[arg(long, default_value_t=String::from( "./" ))]
        artifact_output_directory: String,

        /// Identifier used for the soundtrack in the generated source file
        #[arg(long, default_value_t=String::from( "MuzPlaceholder1" ))]
        soundtrack_label: String,

        /// Identifier used for the soundtrack's artifacts in the generated source file
        #[arg(long, default_value_t=String::from( "MuzInstrumentList" ))]
        instrument_list_label: String
    }
}